/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: VisitorListenerDelegate.java,v d808b6cb5d50 2010/07/10 21:19:33 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.observation;

import javax.annotation.Nonnull;
import it.tidalwave.mobile.util.ProgressListener;
import it.tidalwave.bluebill.observation.Observation;
import it.tidalwave.bluebill.observation.ObservationItem;
import it.tidalwave.bluebill.observation.ObservationSet;
import it.tidalwave.bluebill.observation.ObservationSetVisitor.Listener;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public class VisitorListenerDelegate implements Listener
  {
    private final Listener delegate;
    private final ProgressListener progressListener;
    private int count;

    public VisitorListenerDelegate (final @Nonnull Listener delegate,
                                    final @Nonnull ProgressListener progressListener)
      {
        this.delegate = delegate;
        this.progressListener = progressListener;
      }

    @Nonnull
    public Listener getDelegate()
      {
        return delegate;
      }

    public void begin (final @Nonnull ObservationSet observationSet)
      {
        progressListener.setStepCount(observationSet.find(Observation.class).count()); // FIXME: waste of time
        progressListener.setStep(count = 0);
        delegate.begin(observationSet);
      }

    public void end (final @Nonnull ObservationSet observationSet)
      {
        delegate.end(observationSet);
      }

    public void beginVisit (final @Nonnull Observation observation)
      {
        delegate.beginVisit(observation);
      }

    public void endVisit (final @Nonnull Observation observation)
      {
        delegate.endVisit(observation);
        progressListener.setStep(++count);
      }
      
    public void visit (final @Nonnull ObservationItem item)
      {
        delegate.visit(item);
      }
  }
