/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: TaxonFilter.java,v d808b6cb5d50 2010/07/10 21:19:33 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.taxonomy;

import javax.annotation.Nonnull;
import java.util.Locale;
import org.openide.util.Lookup;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.bluebill.taxonomy.Taxon;
import it.tidalwave.bluebill.taxonomy.Taxonomy.Type;
import it.tidalwave.util.logging.Logger;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public class TaxonFilter
  {
    private static final String CLASS = TaxonFilter.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    @Nonnull
    private final TaxonomyPreferences preferences = Lookup.getDefault().lookup(TaxonomyPreferences.class);

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public boolean accepts (final @Nonnull Taxon taxon, final @Nonnull CharSequence match)
      {
        final NameMatcher nameMatcher = preferences.getNameMatcher();
//        logger.finest("accepts: %s", nameMatcher);

        if (matchesLocalizedNames(taxon, match, nameMatcher))
          {
            return true;
          }

        if (preferences.isScientificNamesRenderingEnabled())
          {
            return matchesScientificName(taxon, match, nameMatcher);
          }

        return false;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private boolean matchesLocalizedNames (final @Nonnull Taxon taxon,
                                           final @Nonnull CharSequence match,
                                           final @Nonnull NameMatcher nameMatcher)
      {
        for (final Locale locale : preferences.getTaxonomyLocales())
          {
            try
              {
                if (nameMatcher.matches(taxon.getDisplayName(locale), match, locale))
                  {
                    return true;
                  }
              }
            catch (NotFoundException e)
              {
                // never mind
              }
          }

        return false;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private boolean matchesScientificName (final @Nonnull Taxon taxon,
                                           final @Nonnull CharSequence match,
                                           final @Nonnull NameMatcher nameMatcher)
      {
        final StringBuilder buffer = new StringBuilder();

        if (taxon.getType() == Type.SPECIES)
          {
            try
              {
                buffer.append(taxon.getParent().getScientificName()).append(" ");
              }
            catch (NotFoundException e)
              {
                // never mind
              }
          }

        buffer.append(taxon.getScientificName());

        return nameMatcher.matches(buffer.toString(), match, Locale.ENGLISH);
      }
  }
